// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "database.h"
#include <sys/wait.h>

static ThreadKey _dbase;
static unsigned _thread = 0;

Database::Database()
{
	database = this;
	if(!getDatabase())
		throw(this);

	if(initdb)
		Exec("admin");
};

Database::~Database()
{
}

PgDatabase *Database::getDatabase(void)
{
	PgDatabase *dbase = (PgDatabase *)_dbase.getKey();
	char conn[128];
	char *err;

	if(!dbase)
	{
		sprintf(conn, "dbname=%s user=%s",
			keyserver.getDatabase(), keyserver.getUsername());
		dbase = new PgDatabase(conn);
		if(!dbase)
			return NULL;
		err = (char *)dbase->ErrorMessage();
		if(!err)
			err = "";
		if(*err)
		{
			slog(SLOG_ERROR) << "connecting; " << err << endl;
			delete dbase;
			return NULL;
		}
		slog(SLOG_DEBUG) << "connecting; thread=" << ++_thread << endl;
		_dbase.setKey(dbase);		
	}
	return dbase;
}

void Database::Exec(const char *scrname)
{
	char *name = keyserver.getScript(scrname);
	char *argv[8];
	int pid, status;
	
	pid = fork();
	if(pid)
	{
		waitpid(pid, &status, 0);
		return;
	}
	
	argv[0] = "psql";
	argv[1] = "-f";
	argv[2] = name;
	argv[3] = "-d";
	argv[4] = (char *)keyserver.getDatabase();
	argv[5] = "-U";
	argv[6] = (char *)keyserver.getUsername();
	argv[7] = NULL;

	execvp(*argv, argv);
	exit(-1);
}		
	
bool Database::Query(const char *q)
{
	ExecStatusType status;
	PgDatabase *dbase = getDatabase();

	if(!dbase)
		return false;

	status = dbase->Exec(q);

	switch(status)
	{
	case PGRES_FATAL_ERROR:
		slog(SLOG_CRITICAL) << "query; " << dbase->ErrorMessage() << endl;
		return false;
	case PGRES_NONFATAL_ERROR:
	case PGRES_BAD_RESPONSE:
		slog(SLOG_ERROR) << "query; " << dbase->ErrorMessage() << endl;
		return false;
	}
	return true;
}	

Database pgsql;
	
















